function [features] = AccelFeature(rawClip,featureType,WINDOWSIZE,TIMESTEP,NUM_FEATURES,firstTimeStamp)
%
% [features] = computeFeatures(clip,featureType,WINDOWSIZE,TIMESTEP,varargin)
% 
% Calculates the accelerometry features for a data clip. Each feature-vector is 
% calculated by sliding a window of size WINDOWSIZE over the data and calculating
% the features every TIMESTEP seconds, taking the timestamps of the stored
% signal as a reference.
% The matrix returned is of format:
% 
% TS1 f1 f2 ... fn
% TS2 f1 f2 ... fn
% ...
% 
% where TS is the timestamp of the feature calculation and n is the feature
% length.
%
% Feature types supported are:     'featureType' [featureLength]
%  'abs' [6]
%  'mean' [6]
%  'var' [6]
%  'mean_var' [12]
%  'acc1_mean' 'acc2_mean' [3]
%  'acc1_var'  'acc2_var' [3]
%  'acc1_mean_var' 'acc2_mean_var' [6]
%  'fft_acc1X' 'fft_acc1Y' 'fft_acc1Z' [WINDOWSIZE]
%  'fft_acc2X' 'fft_acc2Y' 'fft_acc2Z' [WINDOWSIZE]

if not(exist('NUM_FEATURES'))
    NUM_FEATURES = 0;
end


%this is a simple fix so we can use AccelFeatures to compute means and variances on AudioFeatures
if (strcmp(featureType,'audio_mean_var')==1)
    rawClip.acc1 = rawClip.audioFeatures;
    rawClip.acc2 = rawClip.audioFeatures;
end

if exist('firstTimeStamp')
    firstCommonAccTS = firstTimeStamp;
else
    firstCommonAccTS = max(rawClip.acc1(WINDOWSIZE,1),rawClip.acc2(WINDOWSIZE,1));
end
firstCommonAcc1TSIndex = find(rawClip.acc1(:,1) >= firstCommonAccTS,1);
firstCommonAcc2TSIndex = find(rawClip.acc2(:,1) >= firstCommonAccTS,1);
lastCommonAccTS = min(rawClip.acc1(end,1),rawClip.acc2(end,1));
timeStamps = firstCommonAccTS:TIMESTEP:lastCommonAccTS;
features=zeros(length(timeStamps),NUM_FEATURES);
k = 1; acc1_index = 2; acc2_index = 2; 
for TS=timeStamps
    %It is possible that timestamps are not monotonously increasing, hence the max()
    %acc1_index = max(find(rawClip.acc1(:,1) >= TS, 1),WINDOWSIZE);  %this was too slow
    %acc2_index = max(find(rawClip.acc2(:,1) >= TS, 1),WINDOWSIZE);  %this was too slow
    old_acc1_index = acc1_index; old_acc2_index = acc2_index;
    acc1_index = max(find(rawClip.acc1(acc1_index:min(acc1_index+8*WINDOWSIZE,size(rawClip.acc1,1)),1) >= TS, 1)+acc1_index-1,WINDOWSIZE);
    acc2_index = max(find(rawClip.acc2(acc2_index:min(acc2_index+8*WINDOWSIZE,size(rawClip.acc2,1)),1) >= TS, 1)+acc2_index-1,WINDOWSIZE);
    %if (acc1_index < old_acc1_index | acc2_index < old_acc2_index)
    if (isempty(acc1_index) | isempty(acc2_index))
        fprintf('empty (%d) ',old_acc1_index);
        acc1_index = max(find(rawClip.acc1(:,1) >= TS, 1),WINDOWSIZE);  %just in case something goes wrong
        acc2_index = max(find(rawClip.acc2(:,1) >= TS, 1),WINDOWSIZE);
    end
    acc1_window = [acc1_index - WINDOWSIZE + 1 : acc1_index];
    acc2_window = [acc2_index - WINDOWSIZE + 1 : acc2_index];
    switch featureType
        case 'audio_mean_var'
            features(k,:) = [mean(rawClip.audioFeatures(acc1_window,2:end)) var(rawClip.audioFeatures(acc1_window,2:end))];
            
        case 'mean'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) mean(rawClip.acc2(acc2_window,2:4))];
        case 'abs'
            features(k,:) = [abs(rawClip.acc1(acc1_window,2:4)) abs(rawClip.acc2(acc2_window,2:4))];
        case 'var'
            features(k,:) = [var(rawClip.acc1(acc1_window,2:4)) var(rawClip.acc2(acc2_window,2:4))];
        case 'diff'
            features(k,:) = [sum(abs(rawClip.acc1(acc1_window(2:end),2:4)-rawClip.acc1(acc1_window(1:end-1),2:4))) ...
                                  sum(abs(rawClip.acc2(acc2_window(2:end),2:4)-rawClip.acc2(acc2_window(1:end-1),2:4)))];
        case 'mean_diff'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) mean(rawClip.acc2(acc2_window,2:4)) ...
                                  sum(abs(rawClip.acc1(acc1_window(2:end),2:4)-rawClip.acc1(acc1_window(1:end-1),2:4))) ...
                                  sum(abs(rawClip.acc2(acc2_window(2:end),2:4)-rawClip.acc2(acc2_window(1:end-1),2:4)))];
        case 'mean_var'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) mean(rawClip.acc2(acc2_window,2:4)) ...
                                  var(rawClip.acc1(acc1_window,2:4))  var(rawClip.acc2(acc2_window,2:4)) ];
        case 'mean_var_diff'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) mean(rawClip.acc2(acc2_window,2:4)) ...
                                  var(rawClip.acc1(acc1_window,2:4))  var(rawClip.acc2(acc2_window,2:4)) ...
                                  sum(abs(rawClip.acc1(acc1_window(2:end),2:4)-rawClip.acc1(acc1_window(1:end-1),2:4))) ...
                                  sum(abs(rawClip.acc2(acc2_window(2:end),2:4)-rawClip.acc2(acc2_window(1:end-1),2:4)))];
        case 'mean_var_sqr'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) mean(rawClip.acc2(acc2_window,2:4)) ...
                                  var(rawClip.acc1(acc1_window,2:4)).^2  var(rawClip.acc2(acc2_window,2:4)).^2];
        case 'mean_var_var'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) mean(rawClip.acc2(acc2_window,2:4)) ...
                                  var(rawClip.acc1(acc1_window,2:4))  var(rawClip.acc2(acc2_window,2:4))...
                                  var(rawClip.acc1(acc1_window,2:4))  var(rawClip.acc2(acc2_window,2:4))];
        case 'mean_var_sqr1'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) mean(rawClip.acc2(acc2_window,2:4)) ...
                                  var(rawClip.acc1(acc1_window,2:4)).^2  var(rawClip.acc2(acc2_window,2:4))];
        case 'mean_var_var1'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) mean(rawClip.acc2(acc2_window,2:4)) ...
                                  var(rawClip.acc1(acc1_window,2:4))  var(rawClip.acc2(acc2_window,2:4))...
                                  var(rawClip.acc1(acc1_window,2:4))];

        case 'acc1_mean'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4))];
        case 'acc1_var'
            features(k,:) = [var(rawClip.acc1(acc1_window,2:4)) ];
        case 'acc1_mean_var'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) var(rawClip.acc1(acc1_window,2:4))];
%         case 'acc1_mean_var_G'
%             res = mean(rawClip.acc1(acc1_window,2:4)) var(rawClip.acc1(acc1_window,2:4));
%             features = [res sum([0 0 .*res(1:3))];
        
        case 'acc1_mean_cov'
            [coeff,score,latent] = princomp(cov(rawClip.acc1(acc1_window,2:4)));
            [y,i]=sort(-latent);
            latent=latent(i);coeff=coeff(:,i);score=score(:,i);
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) latent(1)*coeff(:,1)'];
        case 'acc2_mean_cov'
            [coeff,score,latent] = princomp(cov(rawClip.acc2(acc2_window,2:4)));
            [y,i]=sort(-latent);
            latent=latent(i);coeff=coeff(:,i);score=score(:,i);
            features(k,:) = [mean(rawClip.acc2(acc2_window,2:4)) latent(1)*coeff(:,1)'];
        case 'mean_cov'
            [coeff,score,latent] = princomp(cov(rawClip.acc1(acc1_window,2:4)));
            [y,i]=sort(-latent);
            latent=latent(i);coeff=coeff(:,i);
            [coeff2,score2,latent2] = princomp(cov(rawClip.acc2(acc2_window,2:4)));
            [y,i]=sort(-latent2);
            latent2=latent2(i);coeff2=coeff2(:,i);
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) latent(1)*coeff(:,1)' mean(rawClip.acc2(acc2_window,2:4)) latent2(1)*coeff2(:,1)'];
        case 'acc1_mean_eig'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) eig(cov(rawClip.acc1(acc1_window,2:4)))'];
        case 'acc2_mean_eig'
            features(k,:) = [mean(rawClip.acc2(acc2_window,2:4)) eig(cov(rawClip.acc2(acc2_window,2:4)))'];
        case 'mean_eig'
            features(k,:) = [mean(rawClip.acc1(acc1_window,2:4)) eig(cov(rawClip.acc1(acc1_window,2:4)))' ...
                                  mean(rawClip.acc2(acc2_window,2:4)) eig(cov(rawClip.acc2(acc2_window,2:4)))'];

        case 'acc2_mean'
            features(k,:) = [mean(rawClip.acc2(acc2_window,2:4))];
        case 'acc2_var'
            features(k,:) = [var(rawClip.acc2(acc2_window,2:4)) ];
        case 'acc2_mean_var'
            features(k,:) = [mean(rawClip.acc2(acc2_window,2:4)) var(rawClip.acc2(acc2_window,2:4))];

        case 'fft_acc1X'
            res = abs(fft(rawClip.acc1(acc1_window,2),WINDOWSIZE))';
            features(k,:) = [res(1:NUM_FEATURES)];
        case 'fft_acc1Y'
            res = abs(fft(rawClip.acc1(acc1_window,3),WINDOWSIZE))';
            features(k,:) = [res(1:NUM_FEATURES)];
        case 'fft_acc1Z'
            res = abs(fft(rawClip.acc1(acc1_window,4),WINDOWSIZE))';
            features(k,:) = [res(1:NUM_FEATURES)];
        case 'fft_acc2X'
            res = abs(fft(rawClip.acc2(acc2_window,2),WINDOWSIZE))';
            features(k,:) = [res(1:NUM_FEATURES)];
        case 'fft_acc2Y'
            res = abs(fft(rawClip.acc2(acc2_window,3),WINDOWSIZE))';
            features(k,:) = [res(1:NUM_FEATURES)];
        case 'fft_acc2Z'
            res = abs(fft(rawClip.acc2(acc2_window,4),WINDOWSIZE))';
            features(k,:) = [res(1:NUM_FEATURES)];
         
        case 'fft_acc1X_noDC'
            res = abs(fft(rawClip.acc1(acc1_window,2),WINDOWSIZE))';
            features(k,:) = [res(2:NUM_FEATURES+1)];
        case 'fft_acc1Y_noDC'
            res = abs(fft(rawClip.acc1(acc1_window,3),WINDOWSIZE))';
            features(k,:) = [res(2:NUM_FEATURES+1)];
        case 'fft_acc1Z_noDC'
            res = abs(fft(rawClip.acc1(acc1_window,4),WINDOWSIZE))';
            features(k,:) = [res(2:NUM_FEATURES+1)];
        case 'fft_acc2X_noDC'
            res = abs(fft(rawClip.acc2(acc2_window,2),WINDOWSIZE))';
            features(k,:) = [res(2:NUM_FEATURES+1)];
        case 'fft_acc2Y_noDC'
            res = abs(fft(rawClip.acc2(acc2_window,3),WINDOWSIZE))';
            features(k,:) = [res(2:NUM_FEATURES+1)];
        case 'fft_acc2Z_noDC'
            res = abs(fft(rawClip.acc2(acc2_window,4),WINDOWSIZE))';
            features(k,:) = [res(2:NUM_FEATURES+1)];
            
        case 'fft_acc12X'
            res1 = abs(fft(rawClip.acc1(acc1_window,2),WINDOWSIZE))';
            res2 = abs(fft(rawClip.acc2(acc2_window,2),WINDOWSIZE))';
            features(k,:) = [res1(1:NUM_FEATURES/2) res2(1:NUM_FEATURES/2)];
        case 'fft_acc12Y'
            res1 = abs(fft(rawClip.acc1(acc1_window,3),WINDOWSIZE))';
            res2 = abs(fft(rawClip.acc2(acc2_window,3),WINDOWSIZE))';
            features(k,:) = [res1(1:NUM_FEATURES/2) res2(1:NUM_FEATURES/2)];
        case 'fft_acc12Z'
            res1 = abs(fft(rawClip.acc1(acc1_window,4),WINDOWSIZE))';
            res2 = abs(fft(rawClip.acc2(acc2_window,4),WINDOWSIZE))';
            features(k,:) = [res1(1:NUM_FEATURES/2) res2(1:NUM_FEATURES/2)];
           
        case 'fft_acc12X_noDC'
            res1 = abs(fft(rawClip.acc1(acc1_window,2),WINDOWSIZE))';
            res2 = abs(fft(rawClip.acc2(acc2_window,2),WINDOWSIZE))';
            features(k,:) = [res1(2:NUM_FEATURES/2+1) res2(2:NUM_FEATURES/2+1)];
        case 'fft_acc12Y_noDC'
            res1 = abs(fft(rawClip.acc1(acc1_window,3),WINDOWSIZE))';
            res2 = abs(fft(rawClip.acc2(acc2_window,3),WINDOWSIZE))';
            features(k,:) = [res1(2:NUM_FEATURES/2+1) res2(2:NUM_FEATURES/2+1)];
        case 'fft_acc12Z_noDC'
            res1 = abs(fft(rawClip.acc1(acc1_window,4),WINDOWSIZE))';
            res2 = abs(fft(rawClip.acc2(acc2_window,4),WINDOWSIZE))';
            features(k,:) = [res1(2:NUM_FEATURES/2+1) res2(2:NUM_FEATURES/2+1)];
            
        case 'fft_acc12X_H'
            res1 = abs(fft(rawClip.acc1(acc1_window,2).*hanning(WINDOWSIZE),WINDOWSIZE))';
            res2 = abs(fft(rawClip.acc2(acc2_window,2).*hanning(WINDOWSIZE),WINDOWSIZE))';
            features(k,:) = [res1(1:NUM_FEATURES/2) res2(1:NUM_FEATURES/2)];
        case 'fft_acc12_mag'
            res1 = abs(fft(sqrt(rawClip.acc1(acc1_window,2).^2+rawClip.acc1(acc1_window,3).^2+rawClip.acc1(acc1_window,4).^2),WINDOWSIZE))';
            res2 = abs(fft(sqrt(rawClip.acc2(acc2_window,2).^2+rawClip.acc2(acc2_window,3).^2+rawClip.acc2(acc2_window,4).^2),WINDOWSIZE))';
            features(k,:) = [res1(2:NUM_FEATURES/2+1) res2(2:NUM_FEATURES/2+1)];
       otherwise,
                error(['invalid feature type ' featureType]);

    end
    k = k+1;
end

%normalizing all features to mean 0 and variance 1
% for i=1:size(features,2)
%     features(:,i) = (features(:,i)-mean(features(:,i)))/sqrt(var(features(:,i)));
% end
%features = (features-ones(size(features,1),1)*mean(features))/sqrt(var(features));

if isempty(features), disp('Warning: Empty featurevector');end;

features = [timeStamps' features];
