function [classifications] = GClassifier(features, bnet)
%
% [classifications] = GClassifier(features, bnet)
%
% Gclassifier performs a GMM classification using the bnet on the provided
% features. The features are expected to be of format
% TS1 f1 f2 .. fn
% TS2 f1 f2 .. fn
%
% The format of classifications is:
%
% TS1 p1 p2 ... pn decision
% TS2 p1 p2 ... pn decision
% ...
%

%%% check if bnet.type field exists
try
    if (~strcmp(bnet.type, 'asdf'))
        foo = 'field existed';
    end
catch
    bnet.type = 'toolbox';    % the bnet.type field is new, old bnets are using the toolbox
end

%%% for the light version and for the toolbox version with single Gaussian
%%% we calculate the decisions 'manually'. That is faster and comparable to
%%% the C-code

if (strcmp(bnet.type,'light'))
    priors = bnet.priors;
    cv = bnet.cv;
    u = bnet.mu;
    singleGaussian = 1;
else
    singleGaussian = length(bnet.CPD)==2;
    if (singleGaussian)
        priors = get_field(bnet.CPD{1},'cpt');
        cv = get_field(bnet.CPD{2},'cov');
        u = get_field(bnet.CPD{2},'mean');
    end
end

if (singleGaussian)
    numFeatures = size(features,2)-1; % index 1 is the timestamp
    numClasses = size(u,2);
    % pre-computation of expensive calculations that are the same for all cases
    for j=1:numClasses
        sigma_det(j) = det(cv(:,:,j));
        inv_cov(:,:,j) = cv(:,:,j)^(-1);
    end
    % for j=1:numClasses       %not done for E-step
    %    cv(:,:,j) = diag(ones(1,numFeatures).*0.01);
    % end
    decision = zeros(size(features,1),1);
    p = zeros(size(features,1),numClasses);
    for i=1:size(features,1)
        summ = 0;
        for j=1:numClasses,
            p(i,j) = priors(j) / sqrt( (2*pi)^numFeatures * sigma_det(j))*exp(-0.5*(features(i,2:end)-u(:,j)')*inv_cov(:,:,j)*(features(i,2:end)-u(:,j)')');
            summ = summ + p(i,j);
        end
        for j=1:numClasses,
            p(i,j) = p(i,j) / summ;
        end
        [foo decision(i,:)] = max(p(i,:));
    end
else
    %%% this is the E-step calculation using the bnet functions %%%

    evidence=cell(3,1);   %% Start out with nothing observed
    decision = zeros(size(features,1),1);
    p = zeros(size(features,1),bnet.node_sizes(1));
    engine = jtree_inf_engine(bnet);
    for i=1:size(features,1)
        evidence{3}=features(i,2:end)';
        [engine2, ll] = enter_evidence(engine,evidence);
        marg = marginal_nodes(engine2,1);
        p(i,:)=marg.T';
        [foo decision(i,:)] = max(p(i,:));
        % mpe = find_mpe(engine, evidence);
        % decision_mpe(i,:) = mpe{1}(1);
    end
end


classifications = [features(:,1) p decision];
