function rawClips = SignalLoader(dataFilePrefix, varargin)
% Version 2.15  by Mark Blum, MIT Media Lab 2005.
% rawClips = SignalLoader(dataFilePrefix, ...)
%
% This function is designed to import clips of sensor data that were recorded
% with SignalRecorder and labeled with the webbased offline Annotation software.
% 
% The files needed look like:
%       - Name_08_32_57.audio
%       - Name_08_32_57.accel1
%       - Name_08_32_57.accel2
%       - labels.txt
% For each line in labels.txt (which includes a timestamp) SignalLoader
% will extract a clip of K seconds length, where K = 'seconds after' -
% 'seconds prior'.
% The output is a one-dimensional cell array, where each entry is a struct
% representing a clip.
%
% Below are the names of optional arguments [default value in brackets].
% Pass as 'PropertyName1', PropertyValue1, 'PropertyName2', PropertyValue2, ...
%
% 'aggregate clips' [false]   : If activated, consective clips with identical
%                               labels are merged to one big clip. Otherwise
%                               seperate clips are created for each label.
% 'label of interest' [0]     : If 'aggregate clips' is set, a 'label of interest' 
%                               can be set to the column containing the label according
%                               to which the clips should be aggregated. Default is 0, 
%                               meaning all labels
% 'seconds prior' [10]        : when to start the clip in respect to the timestamp
% 'seconds After' [20]        : when to stop the clip in respect to the timestamp
% 'clean data' [true]         : Limits the acceleration data to [lower..upper bound]
%                               'lower bound' [170] and 'upper bound' [440]
% 'normalize' [matrix]        : Normalization matrix that is used to shift and scale the 
%                               acceleration data. 
%                               Format:
%                               [muX1 muY1 muZ1 1GX1 1GY1 1GZ1 e.g.[263 274 268 20 20 18
%                                muX2 muY2 muZ2 1GX2 1GY2 1GZ2]     232 234 288 20 21 20]
%                               Pass 0 to skip normalization.
%                               Pass 'f' to read from textfile 'Config_MITes.txt'
% 'oneG_correction' [false]   : in case of nomalization, the Y component of the first 
%                               (hip) accelerometer will be adjusted to "zero G". This 
%                               results in values closer to 0, which is good for EM-learning.
% 'linspace timestamps'[false]: this will recalculate the timestamps of the acceleration
%                               data using linspace(first TS, last TS, numSamples)
% 'preserve timestamps' [fals]: If true, all timestamps remain as read from file. 
%                               Else, the first timestamp is subtracted from all others 
%                               to create nice values, starting from 0
% 'synchronize' [false]       : Acceleration sensors do not sample at exactly the same
%                               rate. This option creates synchronized data between the
%                               first and the last common timestamp with linearly spaced
%                               timestamps using the resample() command
%                               data using linspace(first TS, last TS, numSamples)
% 'online labels' [false]     : Instead of the offline labels in 'labels.txt' the
%                               file '*Prefix*.labels' is used.
% 'clip quality' [0]          : [0] will extract all clips
%                               [1] will extract clips of medium to high quality
%                               [2] will only extract the high quality clips
% 'no audio' [false]          : will not include the audio data for a much smaller file
% 'no acc' [false]            : will not include any acceleration data
% 'no clips' [false]          : This is a simple hack for getting the entire data
%                               as "one big clip"
% 'plots' [true]              : Shows plots of the raw data
% 'save' [false]              : saves the output cell array in the current directory
%
% For more information, read the tuturial on the Wiki:
% http://borglab.media.mit.edu/wiki/borglab/signalrecorder


fprintf('SignalLoader Version 2.15\n');
fprintf('------------------------\n');
startTime = clock;

audioSamplesPerLine = 256;
defSecondsPrior = 10;
defSecondsAfter = 20;
aggregateClips = false;
cleanData = true;
lowerBound = 170;
upperBound = 440;
linspaceTimestamps = false;
synchronize = false;
hasOnlineLabels = false;
labelOfInterest = 0;
labelConfidence = [0 0 0 0];
includeAudio = true;
includeAcc = true;
noClips = false;
plots = true;
saveData = false;
oneG_correction = false;
preserveTimeStamps = false;

if nargin >= 2
    args = varargin;
    nargs = length(args);
    for i=1:2:nargs
        switch args{i},
            case 'aggregate clips',             aggregateClips = args{i+1};
            case 'seconds prior',              defSecondsPrior = args{i+1};
            case 'seconds after',              defSecondsAfter = args{i+1};
            case 'clean data',                       cleanData = args{i+1};
            case 'normalize',                          normMat = args{i+1};
            case 'oneG_correction',            oneG_correction = args{i+1};
            case 'lower bound',                     lowerBound = args{i+1};
            case 'upper bound',                     upperBound = args{i+1};
            case 'save',                              saveData = args{i+1};
            case 'linspace timestamps',     linspaceTimestamps = args{i+1};
            case 'synchronize',                    synchronize = args{i+1};
            case 'online labels',              hasOnlineLabels = args{i+1};
            case 'label of interest',          labelOfInterest = args{i+1};
            case 'label confidence',           labelConfidence = args{i+1};
            case 'no audio',                      includeAudio = not(args{i+1});
            case 'no acc',                          includeAcc = not(args{i+1});
            case 'no clips',                           noClips = args{i+1};
            case 'preserve timestamps',     preserveTimeStamps = args{i+1};
            case 'plots',                                plots = args{i+1};
            case 'audio samples per line', audioSamplesPerLine = args{i+1};
            otherwise,
                error(['invalid argument name ' args{i}]);
        end
    end
end
if (not(includeAudio) & not(includeAcc))
    fprintf('SignalLoader: Error, neither audio nor acceleration was selected for load\n');
    return
end
if (synchronize), linspaceTimestamps = false; end % linear spacing is also done in 'synchronize'
if (noClips), aggregateClips = false; end

temp = strread(dataFilePrefix,'%s','delimiter','\\');
path = ''; for i=1:size(temp,1)-1, path = [path temp{i} '\']; end

%%%%% Load data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (includeAudio)
    disp(sprintf('SignalLoader: loading %s',strcat(dataFilePrefix,'.audio')));
    %%% we load the file by splitting off the time-stamp into a double
    %%% float, and the samples into d16 (signed short) in order to save
    %%% memory. textscan is said to be fast

    fprintf('              Reading TimeStamps and determining number of lines in file: ');
    fid = fopen(strcat(dataFilePrefix,'.audio'),'r');
    audioRaw.timeStamps = cell2num(textscan(fid,'%f64%*[^\n]','delimiter','EOL','bufSize',audioSamplesPerLine*8));
    fclose(fid);
    %whos
    numRows = size(audioRaw.timeStamps,1);
    fprintf('%d\n',numRows);
    fprintf('              Allocating memory (%4.0f MB)\n',numRows*audioSamplesPerLine*2/1000000);
    audioRaw.values = zeros(numRows,audioSamplesPerLine,'int16');
    
    format = '%*s'; %skip timestamp
    for i=1:audioSamplesPerLine,
        format = sprintf('%s %%d16',format);
    end

    fid = fopen(strcat(dataFilePrefix,'.audio'),'r');
    %whos
    ROWS_PER_CHUNK = 20000;
    fprintf('              Reading values: ');
    for i=1:ceil(numRows/ROWS_PER_CHUNK);
        fprintf('%2.0f%% ',((i-1)/ceil(numRows/ROWS_PER_CHUNK)*100));
        audioRaw.values((i-1)*ROWS_PER_CHUNK+1:min(i*ROWS_PER_CHUNK,numRows),:) = cell2mat(textscan(fid,format,ROWS_PER_CHUNK,'bufSize',audioSamplesPerLine*8));
    end
    fprintf('100%% \n');
    fclose(fid);
    %whos
    numAudioSamples = size(audioRaw.timeStamps,1);
end
if (includeAcc)
    fprintf('SignalLoader: loading %s\n',strcat(dataFilePrefix,'.accel1'));
    acc1Raw =  load(strcat(dataFilePrefix,'.accel1'));
    numAcc1Samples  = size(acc1Raw,1);

    fprintf('SignalLoader: loading %s\n',strcat(dataFilePrefix,'.accel2'));
    acc2Raw =  load(strcat(dataFilePrefix,'.accel2'));
    numAcc2Samples  = size(acc2Raw,1);
end
if (hasOnlineLabels)
    labels = load(strcat(dataFilePrefix,'.labels'));
elseif(noClips)
    labels = [0 0 0 0 0 0 0 0 0 0 0];
else
    labels = load([path 'labels.txt']);
end
numLabels = size(labels,1);

%
% save('RawData.mat','audioRaw','acc1Raw','acc2Raw');
% savetime = toc
% load('RawData.mat');
% loadtime = toc

%%%%% set start time to t = 0 %%%%%%%%%%%%%%%%%%%%%%%%%
if (not(preserveTimeStamps))
    if (includeAcc)
        if (includeAudio)
            t0 = min([audioRaw.timeStamps(1,1),acc1Raw(1,1),acc2Raw(1,1)]);
            audioRaw.timeStamps = audioRaw.timeStamps - t0;
        else
            t0 = min([acc1Raw(1,1),acc2Raw(1,1)]);
        end
        acc1Raw(:,1)  = acc1Raw(:,1)  - t0;
        acc2Raw(:,1)  = acc2Raw(:,1)  - t0;
        labels(:,1)   = labels(:,1)   - t0;
        if (plots)
            myPlot(sprintf('Entire Raw Acceleration Data, %s',dataFilePrefix),'samples','',acc1Raw(:,1),acc1Raw(:,2:4),acc2Raw(:,1),acc2Raw(:,2:4));
            legend('AccX_1','AccY_1','AccZ_1','AccX_2','AccY_2','AccZ_2');
        end
    else
        if (includeAudio)
            t0 = audioRaw.timeStamps(1,1);
            audioRaw.timeStamps = audioRaw.timeStamps - t0;
            labels(:,1) = labels(:,1) - t0;
        end
    end
end
%%%%% cleaning %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (cleanData & includeAcc)
    disp('SignalLoader: Cleaning data...');
    count = [0,0,0,0];
    for i=2:4
        idx = find ((acc1Raw(:,i) > upperBound) | (acc1Raw(:,i) < lowerBound));
        count(1) = count(1) + size(idx,1);
        if idx > 0
            if idx(1) == 1, idx(1)= idx(2); end
            if idx(size(idx,1)) == numAcc1Samples,
                idx(size(idx,1))= idx(size(idx,1)-1);
            end
            acc1Raw(idx,i) = (acc1Raw(idx-1,i)+acc1Raw(idx+1,i))/2;
        end
        idx = find ((acc2Raw(:,i) > upperBound) | (acc2Raw(:,i) < lowerBound));
        count(2) = count(2) + size(idx,1);
        if idx > 0
            if idx(1) == 1, idx(1)= idx(2); end
            if idx(size(idx,1)) == numAcc1Samples,
                idx(size(idx,1))= idx(size(idx,1)-1);
            end
            acc2Raw(idx,i) = (acc2Raw(idx-1,i)+acc2Raw(idx+1,i))/2;
        end
    end
    fprintf('              %d samples were cleaned in acc1Raw (%2.2f%%)\n',count(1),100/numAcc1Samples*count(1));
    fprintf('              %d samples were cleaned in acc2Raw (%2.2f%%)\n',count(2),100/numAcc2Samples*count(2));
end
if (plots & includeAcc)
    if (labelOfInterest)
        myPlot(sprintf('Acceleration Data after cleaning, with labels, %s',dataFilePrefix),...
            'samples','',acc1Raw(:,1),acc1Raw(:,2:4),acc2Raw(:,1),acc2Raw(:,2:4),labels(:,1),labels(:,labelOfInterest)*10,'o');
    else
        myPlot(sprintf('Acceleration Data after cleaning, %s',dataFilePrefix),'samples','',acc1Raw(:,1),acc1Raw(:,2:4),acc2Raw(:,1),acc2Raw(:,2:4));
    end
    legend('AccX_1','AccY_1','AccZ_1','AccX_2','AccY_2','AccZ_2');
end
%%%%% clip extraction %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (hasOnlineLabels & includeAcc)
    disp('SignalLoader: Extracting clips from online labels...'); drawnow;
    rawClips = cell(1,numLabels);
    if (includeAudio)
        lastCommonTS = min([audioRaw.timeStamps(end,1) acc1Raw(end,1) acc2Raw(end,1)]);
    else
        lastCommonTS = min([acc1Raw(end,1) acc2Raw(end,1)]);
    end
    labels = [labels; lastCommonTS -1]; %this is to solve the indexing issue with the last clip
    for i=1:numLabels
        rawClips{i}.label = labels(i,:);
        if (includeAudio)
            temp = audioRaw.values(find((audioRaw.timeStamps > labels(i,1)) & (audioRaw.timeStamps < labels(i+1,1))) ,:);
            rawClips{i}.audioStart = audioRaw.timeStamps(find((audioRaw.timeStamps > labels(i,1)),1,'first'));
            rawClips{i}.audioStop = audioRaw.timeStamps(find((audioRaw.timeStamps < labels(i+1,1)),1,'last'));   %Beware: this is not quite true!
            rawClips{i}.audio = reshape(temp',size(temp,1)*audioSamplesPerLine,1);
            rawClips{i}.audioRate = (size(rawClips{i}.audio,1)-audioSamplesPerLine)/(rawClips{i}.audioStop-rawClips{i}.audioStart);
        end
        rawClips{i}.acc1  = acc1Raw(find((acc1Raw(:,1) > labels(i,1)) & (acc1Raw(:,1) < labels(i+1,1))) ,:);
        rawClips{i}.acc2  = acc2Raw(find((acc2Raw(:,1) > labels(i,1)) & (acc2Raw(:,1) < labels(i+1,1))) ,:);
    end
elseif (aggregateClips & includeAcc)
    disp('SignalLoader: Extracting and aggregating clips from offline labels...'); drawnow;
    if (labelOfInterest)
        labelIndex = labelOfInterest;
    else
        labelIndex = [2:size(labels,2)];
    end
    rawClips = cell(0);
    i = 1; k = 1;
    while (i < numLabels)
        if (labelOfInterest)   %Only the label of interest must be of high confidence/quality
            if (labels(i,labelOfInterest+4) < labelConfidence(labelOfInterest-1)),  % to understand + 4 and -1 look at the meaning of each label column
                i = i+1; continue;    % if the inspected label does not meet the quality, the clip is skipped
            end    
        else
            if (find(labels(i,6:9) < labelConfidence)),   % we require each label confidence to be '1'
                i = i+1; continue;    % if the inspected label does not meet the quality, the clip is skipped
            end  
        end
        clipStart = labels(i,1) - defSecondsPrior;
        clipStop  = labels(i,1) + defSecondsAfter;
        if (i < numLabels), 
            % aggregation loop
            while (labels(i,labelIndex) == labels(i+1,labelIndex)) %as long as the next clip has the same label(s)
                if (labelOfInterest)  
                    if (labels(i,labelOfInterest+4) < labelConfidence(labelOfInterest-1)),
                        break; %exit aggregation loop
                    end
                else
                    if (find(labels(i,6:9) < labelConfidence)),
                        break; %exit aggregation loop
                    end
                end
                if (i+1 >= numLabels), break; end % to avoid errors, exit on last clip
                % if we got this far, the next clip is ok to aggregate
                clipStop = labels(i+1,1) + defSecondsAfter;
                i = i+1;
            end
        end
        rawClips{k}.label = labels(i,:);  %beware, if (labelOfInterest), the other labels may be wrong for parts of this clip
        if (includeAudio)
            temp = audioRaw.values(find((audioRaw.timeStamps > clipStart) & (audioRaw.timeStamps < clipStop)) ,:);
            rawClips{k}.audioStart = audioRaw.timeStamps(find((audioRaw.timeStamps > clipStart),1,'first'));
            rawClips{k}.audioStop = audioRaw.timeStamps(find((audioRaw.timeStamps < clipStop),1,'last'));   %Beware: this is not quite true!
            rawClips{k}.audio = reshape(temp',size(temp,1)*audioSamplesPerLine,1);
            rawClips{k}.audioRate = (size(rawClips{k}.audio,1)-audioSamplesPerLine)/(rawClips{k}.audioStop-rawClips{k}.audioStart);
        end
        rawClips{k}.acc1  = acc1Raw(find((acc1Raw(:,1) > clipStart) & (acc1Raw(:,1) < clipStop)) ,:);
        rawClips{k}.acc2  = acc2Raw(find((acc2Raw(:,1) > clipStart) & (acc2Raw(:,1) < clipStop)) ,:);
        k = k+1; i = i+1;
    end
elseif (noClips)
    disp('SignalLoader: Extracting entire data as one clip...'); drawnow;
    rawClips{1} = cell(1);
    if (includeAcc)
        if (includeAudio)
            clipStart = max([audioRaw.timeStamps(1,1),acc1Raw(1,1),acc2Raw(1,1)]);
            clipStop  = max([audioRaw.timeStamps(end,1),acc1Raw(end,1),acc2Raw(end,1)]);
        else
            clipStart = max([acc1Raw(1,1),acc2Raw(1,1)]);
            clipStop  = max([acc1Raw(end,1),acc2Raw(end,1)]);
        end
    else %%only audio
        clipStart = audioRaw.timeStamps(1,1);
        clipStop  = audioRaw.timeStamps(end,1);
    end
    rawClips{1}.label = [0 0 0 0 0 0 0 0 0 0 0];
    if (includeAudio)
        temp = audioRaw.values(find((audioRaw.timeStamps > clipStart) & (audioRaw.timeStamps < clipStop)) ,:);
        rawClips{1}.audioStart = audioRaw.timeStamps(find((audioRaw.timeStamps > clipStart),1,'first'));
        rawClips{1}.audioStop = audioRaw.timeStamps(find((audioRaw.timeStamps < clipStop),1,'last'));   %Beware: this is not quite true!
        rawClips{1}.audio = reshape(temp',size(temp,1)*audioSamplesPerLine,1);
        rawClips{1}.audioRate = (size(rawClips{1}.audio,1)-audioSamplesPerLine)/(rawClips{1}.audioStop-rawClips{1}.audioStart);
    end
    if (includeAcc)
        rawClips{1}.acc1  = acc1Raw(find((acc1Raw(:,1) > clipStart) & (acc1Raw(:,1) < clipStop)) ,:);
        rawClips{1}.acc2  = acc2Raw(find((acc2Raw(:,1) > clipStart) & (acc2Raw(:,1) < clipStop)) ,:);
    end
else
    fprintf('SignalLoader: Extracting %d-second clips from offline labels...\n',defSecondsPrior+defSecondsAfter); drawnow;
    rawClips = {};
    k = 1;
    for i=1:numLabels
        if (labelOfInterest)   %Only the label of interest must be of high confidence/quality
            if (labels(i,labelOfInterest+4) < labelConfidence(labelOfInterest-1)),  % to understand + 4 and -1 look at the meaning of each label column
                i = i+1; continue;    % if the inspected label does not meet the quality, the clip is skipped
            end    
        else
            if (find(labels(i,6:9) < labelConfidence)),   % we require each label confidence to be '1'
                i = i+1; continue;    % if the inspected label does not meet the quality, the clip is skipped
            end  
        end        
        rawClips{k}.label = labels(i,:);
        clipStart = labels(i,1) - defSecondsPrior;
        clipStop  = labels(i,1) + defSecondsAfter;
        if (includeAudio)
            temp = audioRaw.values(find((audioRaw.timeStamps > clipStart) & (audioRaw.timeStamps < clipStop)) ,:);
            rawClips{k}.audioStart = audioRaw.timeStamps(find((audioRaw.timeStamps > clipStart),1,'first'));
            rawClips{k}.audioStop = audioRaw.timeStamps(find((audioRaw.timeStamps < clipStop),1,'last'));   %Beware: this is not quite true!
            rawClips{k}.audio = reshape(temp',size(temp,1)*audioSamplesPerLine,1);
            rawClips{k}.audioRate = (size(rawClips{k}.audio,1)-audioSamplesPerLine)/(rawClips{k}.audioStop-rawClips{k}.audioStart);
        end
        if (includeAcc)
            rawClips{k}.acc1  = acc1Raw(find((acc1Raw(:,1) > clipStart) & (acc1Raw(:,1) < clipStop)) ,:);
            rawClips{k}.acc2  = acc2Raw(find((acc2Raw(:,1) > clipStart) & (acc2Raw(:,1) < clipStop)) ,:);
        end
        k = k+1;
    end
end

%%%%% Enter some additional information %%%%%%%%%%%%%%%%%%%%%%%%%%

numClips = size(rawClips,2);
for i=1:numClips
    if (includeAcc)
        rawClips{i}.numAccSamples = min(size(rawClips{i}.acc1,1),size(rawClips{i}.acc2,1));
    end
    rawClips{i}.dataset = dataFilePrefix;
    rawClips{i}.labelOfInterest = labelOfInterest;
end

%extractionTime = toc
%clear audioRaw acc1Raw acc2Raw;
%audioTime = linspace(rawClips{33}.audioStart,rawClips{33}.audioStop, size(rawClips{33}.audio,1));
%figure;
%plot(audioTime,rawClips{33}.audio(:,1)/2^15);

%%%%% respace timestamps %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% figure;
% plot(rawClips{48}.acc1(1400:1700,1),rawClips{48}.acc1(1400:1700,2:4));
if (linspaceTimestamps)
    disp('SignalLoader: Respacing timestamps...');
    maxDif = 0.1; maxDifIdx = [];
    for i=1:numClips
        temp = linspace(rawClips{i}.acc1(1,1), ...
            rawClips{i}.acc1(size(rawClips{i}.acc1,1),1),size(rawClips{i}.acc1,1))';
        temp2 = max(abs(rawClips{i}.acc1(:,1)-temp));
        if (temp2 > maxDif)
            maxDif = temp2;
            maxDifIdx = i
            jump = rawClips{i}.acc1(find(abs(rawClips{i}.acc1(:,1)-temp) > maxDif*0.9),1)
        end
        rawClips{i}.acc1(:,1) = temp;

        rawClips{i}.acc2(:,1) = linspace(rawClips{i}.acc2(1,1), ...
            rawClips{i}.acc2(size(rawClips{i}.acc2,1),1),size(rawClips{i}.acc2,1));

    end
    display(sprintf('SignalLoader: Largest time shift due to the respacing: %ds in clip %d',maxDif,maxDifIdx));
elseif (synchronize)
    disp('SignalLoader: Synchronizing timestamps...');
    maxDif = 0.0; maxDifIdx = [];
    for i=1:numClips
        firstCommonAccTS = max(rawClips{i}.acc1(1,1),rawClips{i}.acc2(1,1));
        lastCommonAccTS = min(rawClips{i}.acc1(end,1),rawClips{i}.acc2(end,1));
        firstCommonAcc1TSIndex = find(rawClips{i}.acc1(:,1) >= firstCommonAccTS,1);
        firstCommonAcc2TSIndex = find(rawClips{i}.acc2(:,1) >= firstCommonAccTS,1);
        lastCommonAcc1TSIndex = find(rawClips{i}.acc1(:,1) <= lastCommonAccTS,1,'last');
        lastCommonAcc2TSIndex = find(rawClips{i}.acc2(:,1) <= lastCommonAccTS,1,'last');
        numSamples = lastCommonAcc1TSIndex - firstCommonAcc1TSIndex + 1;
        numSamplesAcc2 = lastCommonAcc2TSIndex-firstCommonAcc2TSIndex+1;
        accTime = linspace(firstCommonAccTS, lastCommonAccTS, lastCommonAcc1TSIndex-firstCommonAcc1TSIndex+1)';
        temp = abs(100/numSamples*(numSamplesAcc2)-100);
        if (temp > maxDif)
            maxDif = temp;
            maxDifIdx = i;
        end
        disp(sprintf('              Clip %d: Size %d, Difference in sampling rate was %1.2f%%',i,numSamples,temp));
        rawClips{i}.acc1 = [accTime rawClips{i}.acc1(firstCommonAcc1TSIndex:lastCommonAcc1TSIndex,2:4)];
        rawClips{i}.acc2 = [accTime resample(rawClips{i}.acc2(firstCommonAcc2TSIndex:lastCommonAcc2TSIndex,2:4),numSamples,numSamplesAcc2)];
        rawClips{i}.numAccSamples = numSamples;
    end
    display(sprintf('SignalLoader: Largest difference in sampling rate: %d%% in clip %d',maxDif,maxDifIdx));
end
% figure;
% plot(rawClips{48}.acc1(1400:1700,1),rawClips{3}.acc1(1400:1700,2:4));

%wavplay(rawClips{33}.audio(:,1)/2^15,rawClips{33}.audioRate);

%%%%% Normalization %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Each MITe has it's own mean value for the three dimensions. It can be
% determined by laying the MITe on a table "in all 6 directions" and taking
% the average of the extremas in each dimension.

if (includeAcc)
if (length(normMat)==1 & normMat==0)
    disp('SignalLoader: NOT normalizing');
else
    if (length(normMat)==1 & normMat==1)
        fprintf('SignalLoader: Using default normalization matrix\n');
        normMat = [235.5  284.5  231.5  60  58.5  61       % [muX1 muY1 muZ1 1GX1 1GY1 1GZ1
                   296    310.5  275    60  61    55.5];   %  muX2 muY2 muZ2 1GX2 1GY2 1GZ2]
    end
    if (length(normMat)==1 & normMat=='f')
        calibrationFile = strcat(path,'Config_MITes.txt');
        fprintf('SignalLoader: Reading normalization matrix from file %s\n',calibrationFile);
        try
            normMat = load(calibrationFile);
        catch
            fprintf('SignalLoader: WARNING, could not read file, using default normalization matrix instead!\n');
            normMat = [235.5  284.5  231.5  60  58.5  61       % [muX1 muY1 muZ1 1GX1 1GY1 1GZ1
                       296    310.5  275    60  61    55.5];   %  muX2 muY2 muZ2 1GX2 1GY2 1GZ2]
        end
    end                
    if (size(normMat)~= [2 6])
        disp('SignalLoader: WARNING, normalization matrix has wrong size. Skipping normalization');
    else        
        disp('SignalLoader: Normalizing acceleration data...');
        if (oneG_correction)
            % This is a 1G correction to achieve a [0 0 0] average for the hip-worn accelerometer
            normMat(1,2) = normMat(1,2) + normMat(1,5); 
        end
        for i=1:numClips
            n = size(rawClips{i}.acc1,1);
            rawClips{i}.acc1(:,2:4) = (rawClips{i}.acc1(:,2:4)-[ones(n,1)*normMat(1,1) ones(n,1)*normMat(1,2) ones(n,1)*normMat(1,3)])...
                ./[ones(n,1)*normMat(1,4) ones(n,1)*normMat(1,5) ones(n,1)*normMat(1,6)];
            n = size(rawClips{i}.acc2,1);
            rawClips{i}.acc2(:,2:4) = (rawClips{i}.acc2(:,2:4)-[ones(n,1)*normMat(2,1) ones(n,1)*normMat(2,2) ones(n,1)*normMat(2,3)])...
                ./[ones(n,1)*normMat(2,4) ones(n,1)*normMat(2,5) ones(n,1)*normMat(2,6)];
            rawClips{i}.normMat = normMat;
        end
    end
end
end
%%%%% Save data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (saveData),
    disp('SignalLoader: Saving data in ./rawClips.mat');
    save rawClips.mat rawClips -v6
end

totalRunTime = etime(clock,startTime);
fprintf('SignalLoader finished in %5.0f seconds = %2.2f hours\n',totalRunTime,totalRunTime/60/60);
fprintf('----------------------------------------------------\n');
return;



%%%%%%%%%%%%%%%%%%%%%%%%%
% The stuff down here is from some other playing around.
% --> May still be of interest

%%%%% extract data vectors
% numAudioSamples = size(data{idx_audio},1);
% numAcc1Samples  = size(data{idx_acc1},1);
% numAcc2Samples  = size(data{idx_acc2},1);
% numLabels       = size(data{idx_labels},1);
% firstTimestamp = min([data{1}(1,2) data{2}(1,2) data{3}(1,2)]);
% lastTimestamp  = max([data{idx_audio}(numAudioSamples,2) data{idx_acc1}(numAcc1Samples,2) data{idx_acc2}(numAcc2Samples,2)]);
%
%
% audioTime = linspace(data{idx_audio}(1,2), data{idx_audio}(numAudioSamples,2), numAudioSamples*1024) - firstTimestamp;
% audioRaw = reshape(data{idx_audio}(:,3:1026)',numAudioSamples*1024,1);
% acc1Time = data{idx_acc1}(:,2) - firstTimestamp;
% acc1Raw  = data{idx_acc1}(:,3:5);
% acc2Time = data{idx_acc2}(:,2) - firstTimestamp;
% acc2Raw  = data{idx_acc2}(:,3:5);
% labelTime = data{idx_labels}(:,2) - firstTimestamp;
% labelRaw = data{idx_labels}(:,3);
%
% %%%%% normalize data
% disp('Normalizing data to mean=0 and variance=1');
% audioNormalized = audioRaw./abs(max(abs(audioRaw)));
% acc1Normalized = acc1Raw-ones(numAcc1Samples,1)*mean(acc1Raw);
% acc1Normalized = acc1Normalized./max(abs(max(abs(acc1Normalized))));
% acc2Normalized = acc2Raw-ones(numAcc2Samples,1)*mean(acc2Raw);
% acc2Normalized = acc2Normalized./max(abs(max(abs(acc2Normalized))));
%
% %%%%% compute continuous labels
% %label:            2                   1               3             ...
% %Sensor:     x  x x  x x  x    x   x x xx  x  x   x x xx  x  x  x x  ...
% %newLabels:  0  0 0  2 2  2    2   2 2 00  1  1   1 1 00  3  3  3 3  ...
% %This does not make sense for audio though..
% %Because of the 'unwanted' movement during anotation, we introduce the
% %garbage class for the samples close to a label transition.
% tvgt = 0.4;  %transitionViaGarbageTime [seconds]
% acc1Labels = zeros(numAcc1Samples,1);
% for i=1:numLabels-1
%    acc1Labels(find((acc1Time > labelTime(i)+tvgt) & (acc1Time < labelTime(i+1)-tvgt)),1) = labelRaw(i);
% end
% acc1Labels(find(acc1Time > labelTime(numLabels))) = labelRaw(numLabels);
% acc2Labels = zeros(numAcc2Samples,1);
% for i=1:numLabels-1
%    acc2Labels(find((acc2Time > labelTime(i)+tvgt) & (acc2Time < labelTime(i+1)-tvgt)),1) = labelRaw(i);
% end
% acc2Labels(find(acc2Time > labelTime(numLabels)+tvgt)) = labelRaw(numLabels);
%
%
% %%%%% create one syncronized acceleration data matrix
% firstCommonAccTimestamp = max(acc1Time(1),acc2Time(1));
% lastCommonAccTimestamp = min(acc1Time(numAcc1Samples),acc2Time(numAcc2Samples));
% firstCommonAcc1TimestampIndex = find(acc1Time >= firstCommonAccTimestamp,1);
% firstCommonAcc2TimestampIndex = find(acc2Time >= firstCommonAccTimestamp,1);
% lastCommonAcc1TimestampIndex = find(acc1Time <= lastCommonAccTimestamp,1,'last');
% lastCommonAcc2TimestampIndex = find(acc2Time <= lastCommonAccTimestamp,1,'last');
% numSamples = lastCommonAcc1TimestampIndex - firstCommonAcc1TimestampIndex + 1;
% numSamplesAcc2 = lastCommonAcc2TimestampIndex-firstCommonAcc2TimestampIndex+1;
% accTime = linspace(firstCommonAccTimestamp, lastCommonAccTimestamp, lastCommonAcc1TimestampIndex-firstCommonAcc1TimestampIndex);
% disp(sprintf('Resampling 2nd acceleration signal. Difference in sampling rate was %1.2f%%',abs(100/numSamples*(numSamplesAcc2)-100)));
% acc1respaced = acc1Normalized(firstCommonAcc1TimestampIndex:lastCommonAcc1TimestampIndex,:);
% acc2respaced = resample(acc2Normalized(firstCommonAcc2TimestampIndex:lastCommonAcc2TimestampIndex,:),numSamples,numSamplesAcc2);
%
%
% % Put processed data (pd) into a struct
% pd = struct('acc1x',acc1respaced(:,1),...
%     'acc1y',acc1respaced(:,2),...
%     'acc1z',acc1respaced(:,3),...
%     'acc2x',acc2respaced(:,1),...
%     'acc2y',acc2respaced(:,2),...
%     'acc2z',acc2respaced(:,3),...
%     'labels',acc1Labels(firstCommonAcc1TimestampIndex:lastCommonAcc1TimestampIndex),...
%     'time',acc1Time(firstCommonAcc1TimestampIndex:lastCommonAcc1TimestampIndex),...
%     'numSamples',numSamples,'origin',dataFile);
%
% %%%%% plot normalized data
% % figure;
% % plot(audioTime,audioNormalized,'b');
% % figure;
% % plot(acc1Time,acc1Normalized,'r',acc2Time,acc2Normalized,'g');
% % figure;
% % plot(audioTime,audioNormalized,'b',acc1Time,acc1Normalized,'r',acc2Time,acc2Normalized,'g');
% % figure;
% % plot(acc1Time,acc1Normalized,'b',acc2Time,acc2Normalized,'r',acc1Time,acc1Labels,'k.--',labelTime,labelRaw,'ro');
% % title('Normalized data with labels'); legend('acc1_X','acc1_Y','acc1_Z','acc2_X','acc2_Y','acc2_Z','Labels');
% % t2 = toc
% plot(pd.time,pd.acc1x,'b',pd.time,pd.acc2x,'r',pd.time,pd.labels,'k.--',labelTime,labelRaw,'ro');
% title('Normalized data with labels'); legend('acc1_X','acc2_X','Labels');
% figure;
% plot(audioTime,audioNormalized);
% title('Raw audio'); legend('audio','Labels');
% pause
% wavplay(audioNormalized,8000);
%
%
% %%%%% Save data
% [prefix foo] = strread(dataFile,'%s%s','delimiter','.');
% save([prefix{1} '_pd.mat'],'pd');
% disp(sprintf('Data saved in file %s',[prefix{1} '_pd.mat']));
% %t3 = toc
%
% success = 1;
% return;