function [bnet_trained t_learn] = trainBNET(features, labels, C, varargin)
%
% [bnet_trained t_learn] = trainBNET(features, labels, C, varargin)
%
% Trains a static bayes net using the features and labels provided
% 'C' is a classifier struct that needs to contain the parameters
% C.NUM_CLASSES C.NUM_FEATURES C.MAXITER,C.EPSILON
%
% Below are the names of optional arguments [default value in brackets].
% Pass as 'PropertyName1', PropertyValue1, 'PropertyName2', PropertyValue2, ...
%
% 'toolbox' [true]    : Use the FullBNT toolbox. The case of a single
%                       Gaussian can be trained without toolbox, the result 
%                       turns out to be different however, in many cases
%                       better!!!
% 'numGaussians' [1]  : number of Gaussians for a mixture model
% 'initiate' [0]      : if true, the bnet will be initialized with the 
%                       feature means. This can help EM convergence

toolbox = 1;
numGaussians = 1;
initiate = 0;

bnet_trained = [];
t_learn = [];

if nargin >= 2
    args = varargin;
    nargs = length(args);
    for i=1:2:nargs
        switch args{i},
            case 'toolbox',               toolbox = args{i+1};
            case 'numGaussians',     numGaussians = args{i+1};
            case 'initiate',             initiate = args{i+1};
            otherwise,
                error(['invalid argument name ' args{i}]);
        end
    end
end

if (~toolbox & numGaussians > 1)
    fprintf('Error: Mixture models must be trained using the toolbox!\n');
    return;
end

if (toolbox)
    if (numGaussians ==1)
    dag = [ 0 1 ; 0 0 ];
    discrete_nodes = [1];
    node_sizes = [C.NUM_CLASSES C.NUM_FEATURES];
    bnet = mk_bnet(dag, node_sizes, 'discrete', discrete_nodes);
    bnet.CPD{1} = tabular_CPD(bnet,1);
    tic;
    if (initiate),
        mu = mean(features,1);
        bnet.CPD{2} = gaussian_CPD(bnet, 2, 'mean', mu'*ones(1,C.NUM_CLASSES));
    else
        bnet.CPD{2} = gaussian_CPD(bnet, 2);
    end
    training = cell(2,size(features,1));
    training(2,:) = num2cell(features',1);
    training(1,:) = num2cell(labels',1);
    engine = jtree_inf_engine(bnet);
    %MAXITER=10;       % The number of iterations of EM (max)   global!
    %EPSILON=1e-1000;  % A very small stopping criterion        global!
    disp('Starting training of single Gaussian bnet using learn_params_em()'); drawnow;
    [bnet_trained, ll, engine2] = learn_params_em(engine,training,C.MAXITER,C.EPSILON);
    bnet_trained.type = 'toolbox';
    else  % (numGaussians > 1)
        dag = [ 0 1 1 ; 0 0 1 ; 0 0 0 ];
        discrete_nodes = [1 2];
        nodes = [1 : 3];
        node_sizes = [C.NUM_CLASSES numGaussians C.NUM_FEATURES];
        bnet = mk_bnet(dag, node_sizes, 'discrete', discrete_nodes);
        bnet.CPD{1} = tabular_CPD(bnet,1);
        bnet.CPD{2} = tabular_CPD(bnet,2);
        bnet.CPD{3} = gaussian_CPD(bnet, 3);
        tic;
        training = cell(3,size(features,1));
        training(3,:) = num2cell(features',1);
        training(1,:) = num2cell(labels',1);
        engine = jtree_inf_engine(bnet);
        disp('Starting training of mixture model bnet using learn_params_em()'); drawnow;
        [bnet_trained, ll, engine2] = learn_params_em(engine,training,C.MAXITER,C.EPSILON);
        bnet_trained.type = 'toolbox';
    end
else % (no toolbox)
    fprintf('Computing single Gaussian bnet...'); drawnow;
    bnet_trained.type = 'light';
    bnet_trained.priors = zeros(C.NUM_CLASSES,1);
    bnet_trained.mu = zeros(C.NUM_FEATURES,C.NUM_CLASSES);
    bnet_trained.cv = zeros(C.NUM_FEATURES,C.NUM_FEATURES,C.NUM_CLASSES);
    tic;
    for i=1:C.NUM_CLASSES
        a = features(find(labels==i),:);
        bnet_trained.priors(i) = size(a,1)/size(features,1);
        bnet_trained.mu(:,i) = mean(a,1)';
        bnet_trained.cv(:,:,i) = cov(a);
    end
    fprintf('done\n');
end

t_learn = toc;
fprintf('Learning time was %3.1f seconds\n',t_learn);
drawnow;
%mu = get_field(bnet2.CPD{2},'mean');
%sig = get_field(bnet2.CPD{2},'cov');

% modelX = []; class= cell(2,1); %% Create an empty cell array for observations
% for i=1:length(features)
%     class{1} = labels(i);
%     sample=sample_bnet(bnet_trained,'evidence',class);
%     modelX(i,:)=sample{2}';
% end

% set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
% subplot(2,1,1);
% plot(features);
% title('trainings data');
% subplot(2,1,2);
% plot(modelX);
% title('sampled data from bnet');
