%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Written by Mark Blum (mark@media.mit.edu) in September 2005
%%%
%%% For each category, this script will load specifically extracted 
%%% data, split that into training and testing sets and will train  
%%% and test the Bayes classifier with single Gaussian PDFs.
%%% At the end, data is loaded in which all categories were marked
%%% confident. This set is classified and will be the basis for 
%%% post-classification.

format compact;
close all; clear all; drawnow;
startTime = clock; tic;
warning off all 
saveResults = 1;
DRIVE_LETTER = 'E';
loadNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set2\set2_');
saveNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set3\set3_preclassified_');
modelsOutputPrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set3\models\');

load([loadNamePrefix 'Posture']);
% cut away the "unknown posture" class, it is trained by computer generated data
[a b c] = findLabels(trainingClips,3,0,0);   
trainingClips = trainingClips(c);

%%%%% add a clip with some random values to emulate 'unknown' class
FS = 87.3;
dn.label = [0 0 0 0 0 0];
dn.acc1 = zeros(0,4);
dn.acc2 = zeros(0,4);
dn.dataset = 'Computer generated';
CHUNK = floor(FS*10);
N = 50;
for i=1:N
    fc = abs(randn);
    [b,a]=butter(4,fc/(FS/2));
    scaleX = abs(randn/3+0.5); meanX = randn/3;
    scaleY = abs(randn/3+0.5); meanY = randn/3;
    scaleZ = abs(randn/3+0.5); meanZ = randn/3;
    temp = [randn(CHUNK,1)*scaleX+meanX randn(CHUNK,1)*scaleY+meanY randn(CHUNK,1)*scaleZ+meanZ];
    dn.acc1 = [dn.acc1; linspace((i-1)*CHUNK+1,i*CHUNK,CHUNK)'/FS filter(b,a,temp)];
    fc = abs(randn)/2;
    [b,a]=butter(4,fc/(FS/2));
    scaleX = abs(randn/3+0.5); meanX = randn/3;
    scaleY = abs(randn/3+0.5); meanY = randn/3;
    scaleZ = abs(randn/3+0.5); meanZ = randn/3;
    temp = [randn(CHUNK,1)*scaleX+meanX randn(CHUNK,1)*scaleY+meanY randn(CHUNK,1)*scaleZ+meanZ];
    dn.acc2 = [dn.acc2; linspace((i-1)*CHUNK+1,i*CHUNK,CHUNK)'/FS filter(b,a,temp)];
end
display('done');
cl{1} = dn;
plotClips(cl,'acc1');
trainingClips{length(trainingClips)+1} = dn;


%%%%% Posture Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
instances = findLabels(trainingClips, 3)

C.Posture.name = 'Posture Classifier';
fprintf('################################### %s ###################################\n',C.Posture.name);
C.Posture.LABEL_STRINGS = {'unknown','lie','sit','stand','walk','run','bike'};
C.Posture.NUM_CLASSES = length(C.Posture.LABEL_STRINGS);
C.Posture.INTERESTING_LABEL_IDX = 4;   % with the current labelling categories, this means 'posture'
C.Posture.MAXITER=2;                   % The number of iterations of EM (max)
C.Posture.EPSILON=1e-1000;             % Stopping criterion for the EM
C.Posture.WINDOWSIZE = 128*3;             % [samples] features will be computed over this window
C.Posture.TIMESTEP = 2;                % [seconds] classification is done once every TIMESTEP seconds
C.Posture.TEST_TO_TRAIN_RATIO = 2;
C.Posture.featureType = 'mean_var';
C.Posture.NUM_FEATURES = 12;

features = [];
labels = [];
fprintf('Computing features for clips');
for i=1:length(trainingClips)
    fprintf(' %d',i);
    trainingClips{i}.accFeaturesPosture = AccelFeature(trainingClips{i},C.Posture.featureType,C.Posture.WINDOWSIZE,C.Posture.TIMESTEP,C.Posture.NUM_FEATURES);
    features = [features; trainingClips{i}.accFeaturesPosture(:,2:end)];
    labels = [labels; ones(size(trainingClips{i}.accFeaturesPosture,1),1)*(trainingClips{i}.label(C.Posture.INTERESTING_LABEL_IDX)+1)];
end
fprintf('\n');

TrafoMatrix = computeLDA(features, labels, 3);
if (TrafoMatrix ~= -1), legend(C.Posture.LABEL_STRINGS);  end
TrafoMatrix = computeLDA(features, labels, 2);
if (TrafoMatrix ~= -1), legend(C.Posture.LABEL_STRINGS);  end

% Do the training
trainingX = features((1:C.Posture.TEST_TO_TRAIN_RATIO:end),:);
trainingLabels = labels((1:C.Posture.TEST_TO_TRAIN_RATIO:end),:);
[C.Posture.bnet C.Posture.t_learn]   = trainBNET(trainingX, trainingLabels, C.Posture);
XML_export_GMM([modelsOutputPrefix 'posture_GMM.xml'],C.Posture.bnet,C.Posture.LABEL_STRINGS)
export_C45([modelsOutputPrefix 'posture_c45'],features,labels,C.Posture.LABEL_STRINGS,C.Posture.TEST_TO_TRAIN_RATIO)

fprintf('Classifying clips');
for i=1:length(trainingClips)
    fprintf(' %d',i);
    trainingClips{i}.classificationPosture = [GClassifier(trainingClips{i}.accFeaturesPosture,C.Posture.bnet)...
        ones(size(trainingClips{i}.accFeaturesPosture,1),1)*(trainingClips{i}.label(C.Posture.INTERESTING_LABEL_IDX)+1)];
end
fprintf('\n');
[C.Posture.accuracy C.Posture.confusion] = EvaluateClassifications(trainingClips,'posture',C.Posture.LABEL_STRINGS);
if (saveResults)
    saveName = [saveNamePrefix 'Posture'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'trainingClips','C');
end
fprintf('done!\n\n');
clear trainingClips

%%%% the format of classificationXXX
%
%  TS1 p1 p2 ... pn decision label
%  TS2 p1 p2 ... pn decision label
%  ...
%
load([loadNamePrefix 'Event']);
% cut away the "manipulation event" class, it is trained by computer generated data
[a b c] = findLabels(trainingClips,4,8,0);   
trainingClips = trainingClips(c);

%%%%% Event Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
instances = findLabels(trainingClips, 4)
C.Event.name = 'Event Classifier';
fprintf('################################### %s ###################################\n',C.Event.name);
C.Event.LABEL_STRINGS = {'no event','eating','typing','shaking hands','clapping hands','driving','brushing teeth','doing the dishes'};
C.Event.NUM_CLASSES = length(C.Event.LABEL_STRINGS);;
C.Event.INTERESTING_LABEL_IDX = 5;   % with the current labelling categories, this means 'Event'
C.Event.MAXITER=2;                   % The number of iterations of EM (max)
C.Event.EPSILON=1e-1000;             % Stopping criterion for the EM
C.Event.WINDOWSIZE = 128*3;          % [samples] features will be computed over this window
C.Event.TIMESTEP = 2;                % [seconds] classification is done once every TIMESTEP seconds
C.Event.TEST_TO_TRAIN_RATIO = 2;
C.Event.featureType = 'mean_var';
C.Event.NUM_FEATURES = 12;

features = [];
labels = [];
fprintf('Computing features for clips');
for i=1:length(trainingClips)
    fprintf(' %d',i);
    trainingClips{i}.accFeaturesEvent = AccelFeature(trainingClips{i},C.Event.featureType,C.Event.WINDOWSIZE,C.Event.TIMESTEP,C.Event.NUM_FEATURES);
    features = [features; trainingClips{i}.accFeaturesEvent(:,2:end)];
    labels = [labels; ones(size(trainingClips{i}.accFeaturesEvent,1),1)*(trainingClips{i}.label(C.Event.INTERESTING_LABEL_IDX)+1)];
end
fprintf('\n');
TrafoMatrix = computeLDA(features, labels, 3);
if (TrafoMatrix ~= -1), legend(C.Event.LABEL_STRINGS);  end
TrafoMatrix = computeLDA(features, labels, 2);
if (TrafoMatrix ~= -1), legend(C.Event.LABEL_STRINGS);  end

% Do the training
trainingX = features((1:C.Event.TEST_TO_TRAIN_RATIO:end),:);
trainingLabels = labels((1:C.Event.TEST_TO_TRAIN_RATIO:end),:);
[C.Event.bnet C.Event.t_learn]   = trainBNET(trainingX, trainingLabels, C.Event);
XML_export_GMM([modelsOutputPrefix 'event_GMM.xml'],C.Event.bnet,C.Event.LABEL_STRINGS)
export_C45([modelsOutputPrefix 'event_c45'],features,labels,C.Event.LABEL_STRINGS,C.Event.TEST_TO_TRAIN_RATIO)

fprintf('Classifying clips');
for i=1:length(trainingClips)
    fprintf(' %d',i);
    trainingClips{i}.classificationEvent = [GClassifier(trainingClips{i}.accFeaturesEvent,C.Event.bnet)...
        ones(size(trainingClips{i}.accFeaturesEvent,1),1)*(trainingClips{i}.label(C.Event.INTERESTING_LABEL_IDX)+1)];
end
fprintf('\n');
[C.Event.accuracy C.Event.confusion] = EvaluateClassifications(trainingClips,'event',C.Event.LABEL_STRINGS);
if (saveResults)
    saveName = [saveNamePrefix 'Event'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'trainingClips','C');
end
fprintf('done!\n\n');
%%% take away the random clip again
%trainingClips = trainingClips(1:end-1);
clear trainingClips

%%% Because the EM-algorithms in the BNT toolbox have problems converging
%%% if obervation values are much greater than one, we normalize them.
NORMALIZATION = [1 8.540838e-001 1.451333e+001 4.480513e+000 3.008648e-002 4.305218e+002 1.336457e-001 1.157824e+002 3.351075e-001 6.731011e-003 2.713941e+004];

%%%%% Location Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load([loadNamePrefix 'Location']);

%Exclude the fix for subway and dont know
[a b c] = findLabels(trainingClips,1,0,0);
trainingClips = trainingClips(c);
[a b c] = findLabels(trainingClips,1,5,0);
trainingClips = trainingClips(c);

[a b c] = findLabels(trainingClips,1,9,0);   
for i=1:length(b)
    trainingClips{b(i)}.label(2) = 5;  %relabel restaurant to "5"
end

instances = findLabels(trainingClips, 1)
C.Location.name = 'Location Classifier';
fprintf('################################### %s ###################################\n',C.Location.name);
C.Location.LABEL_STRINGS = {'office','home','outdoors','indoors','restautant','car','street','shop'};
C.Location.NUM_CLASSES = length(C.Location.LABEL_STRINGS);
C.Location.INTERESTING_LABEL_IDX = 2;   % with the current labelling categories, this means 'Location'
C.Location.MAXITER=5;                   % The number of iterations of EM (max)
C.Location.EPSILON=1e-1000;             % Stopping criterion for the EM
C.Location.WINDOWSIZE = 300;            % [samples] features will be computed over this window
C.Location.TIMESTEP = 300/62.5;         % [seconds] classification is done once every TIMESTEP seconds
C.Location.TEST_TO_TRAIN_RATIO = 2;
C.Location.FRAMESIZE = 256;
C.Location.FRAMESTEP = 128;
C.Location.NUM_FEATURES = 10;
C.Location.featureType = 'default';
C.Location.rms = 7.745967e-003;
features = [];
labels = [];
fprintf('Computing features for clips');
for i=1:length(trainingClips)
    fprintf(' %d',i);
    audio = double(trainingClips{i}.audio)/2^15;
    numFrames = floor( (length(audio) - (C.Location.FRAMESIZE-C.Location.FRAMESTEP)) / C.Location.FRAMESTEP );
    featureTimestamps = linspace(trainingClips{i}.audioStart,trainingClips{i}.audioStop,numFrames)';
    features_temp = speech_features(audio,C.Location.FRAMESIZE,C.Location.FRAMESTEP,C.Location.rms/5)';
    trainingClips{i}.audioFeatures = [featureTimestamps features_temp(:,[4 6 3 7 1])]; % same order of features as in c-code
    trainingClips{i}.audioFeaturesLocation = AccelFeature(trainingClips{i},'audio_mean_var',C.Location.WINDOWSIZE,C.Location.TIMESTEP,C.Location.NUM_FEATURES);
    trainingClips{i}.audioFeaturesLocation = trainingClips{i}.audioFeaturesLocation ./ (ones(size(trainingClips{i}.audioFeaturesLocation,1),1)*NORMALIZATION);
    trainingClips{i}.audioFeatures = [];  % free memory
    features = [features ; trainingClips{i}.audioFeaturesLocation(:,2:end)];
    labels = [labels ; ones(size(trainingClips{i}.audioFeaturesLocation,1),1)*(trainingClips{i}.label(C.Location.INTERESTING_LABEL_IDX))];
end
fprintf('\n'); clear audio clip;

TrafoMatrix = computeLDA(features, labels, 3);
if (TrafoMatrix ~= -1), legend(C.Location.LABEL_STRINGS);  end
TrafoMatrix = computeLDA(features, labels, 2);
if (TrafoMatrix ~= -1), legend(C.Location.LABEL_STRINGS);  end

trainingX = features((1:C.Location.TEST_TO_TRAIN_RATIO:end),:);
trainingLabels = labels((1:C.Location.TEST_TO_TRAIN_RATIO:end),:);
[C.Location.bnet C.Location.t_learn]   = trainBNET(trainingX, trainingLabels, C.Location);
XML_export_GMM([modelsOutputPrefix 'location_GMM.xml'],C.Location.bnet,C.Location.LABEL_STRINGS)
export_C45([modelsOutputPrefix 'location_c45'],features,labels,C.Location.LABEL_STRINGS,C.Location.TEST_TO_TRAIN_RATIO)

fprintf('Classifying clips');
for i=1:length(trainingClips)
    fprintf(' %d',i);
    trainingClips{i}.classificationLocation = [GClassifier(trainingClips{i}.audioFeaturesLocation,C.Location.bnet)...
        ones(size(trainingClips{i}.audioFeaturesLocation,1),1)*(trainingClips{i}.label(C.Location.INTERESTING_LABEL_IDX))];
end
fprintf('\n');
[C.Location.accuracy C.Location.confusion] = EvaluateClassifications(trainingClips,'location',C.Location.LABEL_STRINGS);
if (saveResults)
    saveName = [saveNamePrefix 'Location'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'trainingClips','C');
end
fprintf('done!\n\n');
if (saveResults)
    saveName = [saveNamePrefix 'C'];
    fprintf('Saving classifers as %s.mat...',saveName);
    save(saveName,'C');
end
fprintf('done!\n');
clear trainingClips

%%%%% Speech Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load([loadNamePrefix 'Speech']);

%append laughter clip
dn.label = [0 0 6 0 0 0 1 0 0 0 0];
%dn.acc1 = zeros(0,4); dn.acc2 = zeros(0,4);
dn.dataset = 'Computer generated';
dn.audio = int16(wavread(strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\laughter.wav'))*25000); %not 32000 because the energy level of the wav-file is quite high
dn.audioStart = 0;
dn.audioStop = length(dn.audio)/8000;
trainingClips{length(trainingClips)+1} = dn;

instances = findLabels(trainingClips, 2)
C.Speech.name = 'Speech Classifier';
fprintf('################################### %s ###################################\n',C.Speech.name);
C.Speech.LABEL_STRINGS = {'no speech','me speaking','other speaker','distant voices','loud crowd','laughter'};
C.Speech.NUM_CLASSES = length(C.Speech.LABEL_STRINGS);
C.Speech.INTERESTING_LABEL_IDX = 3;   % with the current labelling categories, this means 'Speech'
C.Speech.MAXITER=5;                   % The number of iterations of EM (max)
C.Speech.EPSILON=1e-1000;             % Stopping criterion for the EM
C.Speech.WINDOWSIZE = 300;            % [samples] features will be computed over this window
C.Speech.TIMESTEP = C.Speech.WINDOWSIZE/62.5;      % [seconds] classification is done once every TIMESTEP seconds
C.Speech.TEST_TO_TRAIN_RATIO = 2;
C.Speech.FRAMESIZE = 256;
C.Speech.FRAMESTEP = 128;
C.Speech.NUM_FEATURES = 10;
C.Speech.featureType = 'default';
C.Speech.rms = 7.745967e-003;
features = [];
labels = [];
fprintf('Computing features for clips');
for i=1:length(trainingClips) 
    fprintf(' %d',i);
    audio = double(trainingClips{i}.audio)/2^15;
    numFrames = floor( (length(audio) - (C.Speech.FRAMESIZE-C.Speech.FRAMESTEP)) / C.Speech.FRAMESTEP );
    featureTimestamps = linspace(trainingClips{i}.audioStart,trainingClips{i}.audioStop,numFrames)';
    features_temp = speech_features(audio,C.Speech.FRAMESIZE,C.Speech.FRAMESTEP,C.Speech.rms/5)';
    trainingClips{i}.audioFeatures = [featureTimestamps features_temp(:,[4 6 3 7 1])]; % same order of features as in c-code
    trainingClips{i}.audioFeaturesSpeech = AccelFeature(trainingClips{i},'audio_mean_var',C.Speech.WINDOWSIZE,C.Speech.TIMESTEP,C.Speech.NUM_FEATURES);
    trainingClips{i}.audioFeaturesSpeech = trainingClips{i}.audioFeaturesSpeech ./ (ones(size(trainingClips{i}.audioFeaturesSpeech,1),1)*NORMALIZATION);
    trainingClips{i}.audioFeatures = [];  % free memory
    features = [features ; trainingClips{i}.audioFeaturesSpeech(:,2:end)];
    labels = [labels ; ones(size(trainingClips{i}.audioFeaturesSpeech,1),1)*(trainingClips{i}.label(C.Speech.INTERESTING_LABEL_IDX))];
end
fprintf('\n'); clear audio clip;

TrafoMatrix = computeLDA(features, labels, 3);
if (TrafoMatrix ~= -1), legend(C.Speech.LABEL_STRINGS);  end
TrafoMatrix = computeLDA(features, labels, 2);
if (TrafoMatrix ~= -1), legend(C.Speech.LABEL_STRINGS);  end


trainingX = features((1:C.Speech.TEST_TO_TRAIN_RATIO:end),:);
trainingLabels = labels((1:C.Speech.TEST_TO_TRAIN_RATIO:end),:);
[C.Speech.bnet C.Speech.t_learn]   = trainBNET(trainingX, trainingLabels, C.Speech);
XML_export_GMM([modelsOutputPrefix 'speech_GMM.xml'],C.Speech.bnet,C.Speech.LABEL_STRINGS)
export_C45([modelsOutputPrefix 'speech_c45'],features,labels,C.Speech.LABEL_STRINGS,C.Speech.TEST_TO_TRAIN_RATIO)

fprintf('Classifying clips');
for i=1:length(trainingClips)
    fprintf(' %d',i);
    trainingClips{i}.classificationSpeech = [GClassifier(trainingClips{i}.audioFeaturesSpeech,C.Speech.bnet)...
        ones(size(trainingClips{i}.audioFeaturesSpeech,1),1)*(trainingClips{i}.label(C.Speech.INTERESTING_LABEL_IDX))];
end
fprintf('\n');
[C.Speech.accuracy C.Speech.confusion] = EvaluateClassifications(trainingClips,'speech',C.Speech.LABEL_STRINGS);
speakingConfusion = [C.Speech.confusion(1,1) sum(C.Speech.confusion(1,2:end)); sum(C.Speech.confusion(2:end,1)) sum(sum(C.Speech.confusion(2:end,2:end)))]
if (saveResults)
    saveName = [saveNamePrefix 'Speech'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'trainingClips','C');
end
fprintf('done!\n\n');
clear trainingClips dn

fprintf('################################### Classify testClips ###################################\n');
load([loadNamePrefix 'TestClips']);
[a b c] = findLabels(testClips,4,8,0);   
testClips = testClips(c);

%Exclude the fix for subway and dont know
[a b c] = findLabels(testClips,1,0,0);
testClips = testClips(c);
[a b c] = findLabels(testClips,1,5,0);
testClips = testClips(c);

[a b c] = findLabels(testClips,1,9,0);   
for i=1:length(b)
    testClips{b(i)}.label(2) = 5;  %relabel restaurant to "5"
end

fprintf('Total number of TestClips: %d\n',length(testClips));
locationLabels = []; speechLabels = []; postureLabels = []; eventLabels = [];
locationFeatures = []; postureFeatures = []; eventFeatures = []; 
fprintf('Computing features for testClips');
for i=1:length(testClips)
    fprintf(' %d',i);
    %%% Location
    audio = double(testClips{i}.audio)/2^15;
    numFrames = floor( (length(audio) - (C.Location.FRAMESIZE-C.Location.FRAMESTEP)) / C.Location.FRAMESTEP );
    featureTimestamps = linspace(testClips{i}.audioStart,testClips{i}.audioStop,numFrames)';
    features_temp = speech_features(audio,C.Location.FRAMESIZE,C.Location.FRAMESTEP,C.Location.rms/5)';
    testClips{i}.audioFeatures = [featureTimestamps features_temp(:,[4 6 3 7 1])]; % same order of features as in c-code
    testClips{i}.audioFeaturesLocation = AccelFeature(testClips{i},'audio_mean_var',C.Location.WINDOWSIZE,C.Location.TIMESTEP,C.Location.NUM_FEATURES);
    testClips{i}.audioFeaturesLocation = testClips{i}.audioFeaturesLocation ./ (ones(size(testClips{i}.audioFeaturesLocation,1),1)*NORMALIZATION);
    testClips{i}.audioFeatures = [];  % free memory
    locationFeatures = [locationFeatures ; testClips{i}.audioFeaturesLocation(:,2:end)];
    locationLabels = [locationLabels ; ones(size(testClips{i}.audioFeaturesLocation,1),1)*(testClips{i}.label(C.Location.INTERESTING_LABEL_IDX))];
    %%% Speech
    testClips{i}.audioFeaturesSpeech = testClips{i}.audioFeaturesLocation;
    speechLabels = [speechLabels ; ones(size(testClips{i}.audioFeaturesSpeech,1),1)*(testClips{i}.label(C.Speech.INTERESTING_LABEL_IDX))];
    %%% Posture
    testClips{i}.accFeaturesPosture = AccelFeature(testClips{i},C.Posture.featureType,C.Posture.WINDOWSIZE,C.Posture.TIMESTEP, C.Posture.NUM_FEATURES);
    postureFeatures = [postureFeatures; testClips{i}.accFeaturesPosture];
    postureLabels = [postureLabels; ones(size(testClips{i}.accFeaturesPosture,1),1)*(testClips{i}.label(C.Posture.INTERESTING_LABEL_IDX)+1)];
    %%% Event
    testClips{i}.accFeaturesEvent = AccelFeature(testClips{i},C.Event.featureType,C.Event.WINDOWSIZE,C.Event.TIMESTEP, C.Event.NUM_FEATURES);
    eventFeatures = [eventFeatures; testClips{i}.accFeaturesEvent];
    eventLabels = [eventLabels; ones(size(testClips{i}.accFeaturesEvent,1),1)*(testClips{i}.label(C.Event.INTERESTING_LABEL_IDX)+1)];
end
fprintf('\n'); clear audio clip;

% TrafoMatrix = computeLDA(locationFeatures, locationLabels, 3);
% if (TrafoMatrix ~= -1), legend(C.Location.LABEL_STRINGS);  end
% TrafoMatrix = computeLDA(locationFeatures, speechLabels, 3);
% if (TrafoMatrix ~= -1), legend(C.Speech.LABEL_STRINGS);  end
% TrafoMatrix = computeLDA(postureFeatures, postureLabels, 3);
% if (TrafoMatrix ~= -1), legend(C.Posture.LABEL_STRINGS);  end
% TrafoMatrix = computeLDA(eventFeatures, eventLabels, 3);
% if (TrafoMatrix ~= -1), legend(C.Event.LABEL_STRINGS);  end

fprintf('Classifying testClips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.classificationLocation = [GClassifier(testClips{i}.audioFeaturesLocation,C.Location.bnet)...
        ones(size(testClips{i}.audioFeaturesLocation,1),1)*(testClips{i}.label(C.Location.INTERESTING_LABEL_IDX))];
    testClips{i}.classificationSpeech = [GClassifier(testClips{i}.audioFeaturesSpeech,C.Speech.bnet)...
        ones(size(testClips{i}.audioFeaturesSpeech,1),1)*(testClips{i}.label(C.Speech.INTERESTING_LABEL_IDX))];
    testClips{i}.classificationPosture = [GClassifier(testClips{i}.accFeaturesPosture,C.Posture.bnet)...
        ones(size(testClips{i}.accFeaturesPosture,1),1)*(testClips{i}.label(C.Posture.INTERESTING_LABEL_IDX)+1)];
    testClips{i}.classificationEvent = [GClassifier(testClips{i}.accFeaturesEvent,C.Event.bnet)...
        ones(size(testClips{i}.accFeaturesEvent,1),1)*(testClips{i}.label(C.Event.INTERESTING_LABEL_IDX)+1)];
end
fprintf('\n');
[accuracies] = EvaluateAllClassifications(testClips,C);


%%%%% Save classified clips %%%%%%%%%%%%%%%%%%%%%%
if (saveResults)
    saveName = [saveNamePrefix 'testClips'];
    fprintf('Saving classified test clips as %s.mat...',saveName);
    save(saveName,'testClips');
end
fprintf('done!\n');

%%%%% cut away raw data for post classification analysis %%%%%%%%%%%%%%%%%%%%%%
for i=1:length(testClips)
    testClips{i}.audio = [];
    testClips{i}.acc1 = [];
    testClips{i}.acc2 = [];
    testClips{i}.audioFeaturesLocation = [];
    testClips{i}.audioFeaturesSpeech = [];
    testClips{i}.accFeaturesPosture= [];
    testClips{i}.accFeaturesEvent = [];
end
if (saveResults)
    saveName = [saveNamePrefix 'testClips_classifications_only'];
    fprintf('Saving classified test clips (classification only) as %s.mat...',saveName);
    save(saveName,'testClips');
end
fprintf('done!\n');

totalRunTime = etime(clock,startTime);
fprintf('Total runtime was %5.0f seconds = %2.2f hours\n',totalRunTime,totalRunTime/60/60);